--[[****************************************************************
	QuestItems v1.02

	Author: Evil Duck
	****************************************************************

	Description:
		Type /qi or /questitems for brief help

	****************************************************************]]

-- 1.02 Added coloured frames for item status, bugfix for delivered quest, bugfix for quest update
-- 1.00 First version


QUESTITEMS_VERSIONTEXT = "QuestItems v1.02";
SLASH_QUESTITEMS1 = "/questitems";
SLASH_QUESTITEMS2 = "/qi";
local QUESTITEMSBUTTON="QuestItemsItem";
local QUESTITEMSSPACE=3;
local QUESTITEMSSIZE=25;
local QI_FONTHEIGHT=14;
local QUESTITEMSOFFSET=16;
local QUESTITEMS_SCANTIME=1.0;
local QIWAITERTIME=10;
local QIUPDATEDELAY=0.005;
local QUESTITEMS_FRAMETIME=4;
local QUESTITEMS_FADETIME=1;
local QUESTITEMS_DEFAULTICON="Interface\\Icons\\INV_Misc_QuestionMark";


QuestItems={
};
QuestItemsOptions={
	FontSize=QI_FONTHEIGHT,
	Space=QUESTITEMSSPACE,
	Size=QUESTITEMSSIZE,
	Offset=QUESTITEMSOFFSET,
	Height=2,
	Width=4,
};
local lQI={
	Waiter=QIWAITERTIME,
	BGUpdate = {
		last=nil,
		xdir=0,
		xpos=0,
		ydir=0,
		ypos=0
	},
	FrameTimer=-1,
	FrameFaded=nil,
	Quest={
		Rescan=-1,
	},
};

local FRM_GREEN="m060";
local FRM_YELLOW="m120";
local FRM_COPPER="m150";
local FRM_CYAN="p000";
local FRM_BLUE="p060";
local FRM_PURPLE="p090";
local FRM_PINK="p120";
local FRM_RED="p180";



-- Set up for handling
function QuestItems:OnLoad()
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("CHAT_MSG_SYSTEM");
	this:RegisterEvent("BAG_UPDATE");
	SlashCmdList["QUESTITEMS"]=function(msg) self:Slasher(msg) end;
	self:GetButton(0,0);
	this:RegisterForDrag("LeftButton");
end

-- An event has been received
function QuestItems:OnEvent(event)
	if (event=="BAG_UPDATE") then
		self:UpdateItems();
		return;
	end

	if (event=="CHAT_MSG_SYSTEM") then
		local text=arg1;
		local mask=ERR_QUEST_ACCEPTED_S; mask=string.gsub(mask,"%%s","(.+)");
		local questname=text:match(mask);
		if (questname) then lQI.Quest.Rescan=3; end
		return;
	end

	if (event=="VARIABLES_LOADED") then
		QuestItems:GetButton(0,0,false,true);
		if (not QuestItems_Frame0:IsVisible()) then self:ShowMainFrame(); end
		self:SetSize(QuestItemsOptions.Size);
		self:Lock(QuestItemsOptions.Locked);
		self:CreateButtons(QuestItemsOptions.Width,QuestItemsOptions.Height);
		self:ShowMainFrame();
		lQI.Quest.Rescan=5;
	end
end

function QuestItems:TraverseIcons(func,notime)
	local now=time();

	-- Center, shift from right, aim up
	local xdir=1;
	local xpos=0;
	local ydir=1;
	local ypos=0;

	while (xdir~=0 or ydir~=0) do
		while (self:GetButton(xpos,ypos,true)) do			-- Do not create if non-existent
			local button=self:GetButton(xpos,ypos,true);
			if (not button.LastUpdated or button.LastUpdated~=now or notime) then
				if (not notime) then button.LastUpdated=now; end
				func(self,button);
			end
			xpos=xpos+xdir;
		end
		-- Check end of direction
			if (xpos==0 and xdir== 1 and ydir== 1) then xpos=-1; xdir=-1; ypos= 0; ydir= 1;	-- One left to center, shift from left, aim up
		elseif (xpos==0 and xdir==-1 and ydir== 1) then xpos= 0; xdir= 1; ypos=-1; ydir=-1;	-- One below center, shift from right, aim down
		elseif (xpos==0 and xdir== 1 and ydir==-1) then xpos= 0; xdir=-1; ypos=-1; ydir=-1;	-- One below center, shift from left, aim down
		elseif (xpos==0 and xdir==-1 and ydir==-1) then xdir= 0; ydir= 0; break;			-- Done
		else xpos=0; ypos=ypos+ydir; end													-- Continue
	end
end

function QuestItems:FindItemButtonFound(button)
	if (lQI.LF_Button) then return; end
	if (button.deleted) then return; end
	if (not button.User) then return; end
	if (not button.User.Name) then return; end
	if (button.User.Name==lQI.LF_Item) then
		lQI.LF_Button=button;
	end
end
function QuestItems:FindItemButton(item)
	lQI.LF_Button=nil;
	lQI.LF_Item=item;
	self:TraverseIcons(self.FindItemButtonFound,true);
	return lQI.LF_Button;
end
function QuestItems:ClearItemButton(button)
	if (not button and self) then button=self; end	-- If called by xxx.CIB
	button.User=nil;
	button.deleted=nil;
	QuestItems:SetNumber(button,"OurStock",nil);
	QuestItems:SetNumber(button,"MyCount",nil);
	getglobal(button:GetName().."IconTexture"):SetTexture(QUESTITEMS_DEFAULTICON);
	getglobal(button:GetName().."IconTextureOverlay"):SetTexture("");
end

function QuestItems:WriteData(button,lower,upper)
	if (not lower) then lower=""; end
	if (not upper) then upper=""; end
	QuestItems:SetNumber(button,"MyCount",lower,1,1,1);
	QuestItems:SetNumber(button,"OurStock",upper,1,1,1);
end

-- Some cause for update
function QuestItems:UpdateButton(button)
	if (not button) then return; end
	if (button.deleted) then return; end
	if (not lQI.Quest.ItemList) then return; end
	if (not button.User) then
		for item,iEntry in pairs(lQI.Quest.ItemList) do
--			if (GetItemInfo(item)) then									-- Placable
			if (GetItemIcon(item)) then									-- Placable
				if (not self:FindItemButton(item)) then			-- Not placed
--					local _,_,_,_,_,_,_,_,_,texture=GetItemInfo(item);
					local texture=GetItemIcon(item);
					button.User={
						Name=item,
						Texture=texture,
					};
					break;
				end
			end
		end
	end
	if (not button.User) then return; end

	local texture=getglobal(button:GetName().."IconTexture");
	texture:SetTexture(button.User.Texture);				-- Set texture from item
	local amount=GetItemCount(button.User.Name,true);	-- Get local count (bags/bank only)
	if (not button.User.Amount) then			-- Nothing registered here yet
		if (amount>0) then						-- There's something in bags
			button.User.Amount=amount;			-- Save it
		end
	elseif (amount==0) then						-- Was something, now zero
		lQI.Quest.Rescan=3;			-- So rebuild everything
	end
	QuestItems:WriteData(button,amount);

	-- Traverse all quests
	local itemQuests=0;		-- Quests that need this item
	local doneitem=0;		-- Quests that has enough items
	local doneQuests=0;		-- Quests that has all items
	for quest,qTable in pairs(lQI.Quest.List) do
		-- Traverse all items in said quest
		for item,need in pairs(qTable) do
			if (item==button.User.Name) then						-- Found the item in said quest
				itemQuests=itemQuests+1;							-- One more quest
				local amount=GetItemCount(button.User.Name,true);	-- Get local count (bags+bank)
				if (amount>=need) then doneitem=doneitem+1; end		-- Done this item this quest
				-- Traverse all other items in said quest
				local allItems=0;
				local allDone=0;
				for oItem,oNeed in pairs(qTable) do
					if (oItem~="<SECTION>") then
						local oAmount=GetItemCount(oItem,true);	-- Including bank
						if (oAmount>=oNeed) then allDone=allDone+1; end
						allItems=allItems+1;
					end
				end
				if (allItems==allDone) then doneQuests=doneQuests+1; end
			end
		end
	end
	local oTexture="";
	if (itemQuests>0) then						-- There are quests for this item
		if (doneQuests==itemQuests) then		-- All quests have all items
			oTexture="Interface\\AddOns\\QuestItems\\Images\\Frame_"..FRM_GREEN;
		elseif (doneQuests>0) then				-- Some quests have all items
			oTexture="Interface\\AddOns\\QuestItems\\Images\\Frame_"..FRM_YELLOW;
		elseif (doneitem>0) then				-- Some quests have all of this item
			oTexture="Interface\\AddOns\\QuestItems\\Images\\Frame_"..FRM_CYAN;
		end
	end
	getglobal(button:GetName().."IconTextureOverlay"):SetTexture(oTexture);
end

-- Tooltip should be shown
function QuestItems:Tooltip(button)
	local White="|cFFFFFFFF";
	local Yellow="|cFFFFFF00";
	local Red="|cFFFF0000";
	local Green="|cFF00FF00";
	local bBlue="|cFFA0A0FF";		-- Bright blue

	GameTooltip:SetOwner(button,"ANCHOR_RIGHT");
	if (not button.User or not button.User.Name) then
		GameTooltip:SetText(Yellow.."No item in this slot|r");
	else
		GameTooltip:SetText(Yellow..button.User.Name.."|r");
		for quest,qTable in pairs(lQI.Quest.List) do
			for item,iEntry in pairs(qTable) do
				if (item==button.User.Name) then
					GameTooltip:AddLine(White.."Need "..bBlue..iEntry..White.." for quest "..bBlue..quest..White.." in "..qTable["<SECTION>"],1,1,1);
					for oItem,oiEntry in pairs(qTable) do
						if (oItem~=button.User.Name and oItem~="<SECTION>") then
							local currentcount=GetItemCount(oItem,true);	-- Including bank
							local numColour=Green;
							if (currentcount<oiEntry) then numColour=Red; end
							GameTooltip:AddLine(White.."   + Have "..numColour..GetItemCount(oItem,true).." of "..oiEntry..White.." "..oItem,1,1,1);
						end
					end
				end
			end
		end
	end
	GameTooltip:Show();
end

function QuestItems:ScanQuests()
	lQI.Quest.List=nil;
	lQI.Quest.List={};

	ExpandQuestHeader(0);			-- Expand all quest-headers
	local lastHeader=nil;
	local i=1;
	local count=0;
	while (GetQuestLogTitle(i)~=nil) do
		local questTitle,level,questTag,suggestedGroup,isHeader,isCollapsed,isComplete,isDaily=GetQuestLogTitle(i);
		if (isHeader) then
			lastHeader=questTitle;
		else
			count=count+1;
			SelectQuestLogEntry(i);
			if (not lQI.Quest.List[questTitle]) then lQI.Quest.List[questTitle]={}; end
			lQI.Quest.List[questTitle]["<SECTION>"]=lastHeader;
			local goal=GetNumQuestLeaderBoards();
			if (goal and goal>0) then
				local peek=1;
				local itemIndex=1;
				while(peek<=goal) do
					local desc,oType,done=GetQuestLogLeaderBoard(peek);
					if (oType=="item") then
						local _,_,itemName,numItems,numNeeded=string.find(desc, "(.*):%s*([%d]+)%s*/%s*([%d]+)");
						if (itemName) then
							lQI.Quest.List[questTitle][itemName]=tonumber(numNeeded);
						end
						itemIndex=itemIndex+1;
					end
					peek=peek+1;
				end
			end
		end
		i=i+1;
	end

	-- Make the distilled item-list
	lQI.Quest.ItemList=nil;
	lQI.Quest.ItemList={};
	for quest,qTable in pairs(lQI.Quest.List) do
		for item,iEntry in pairs(qTable) do
			if (item~="<SECTION>") then
				lQI.Quest.ItemList[item]=true;					-- Can display if needed
			end
		end
	end
end

function QuestItems:OnUpdate(elapsed)
	if (lQI.Waiter>0) then
		lQI.Waiter=lQI.Waiter-elapsed;
		return;
	end

	if (lQI.Quest.Rescan>0) then
		lQI.Quest.Rescan=lQI.Quest.Rescan-elapsed;
		if (lQI.Quest.Rescan<=0) then
			QuestItems:WipeInterface();
			QuestItems:ScanQuests();
			QuestItems:UpdateItems();
		end
		return;
	end

	if (lQI.BGUpdate.last) then
		lQI.BGUpdate.last=lQI.BGUpdate.last+elapsed;
		if (lQI.BGUpdate.last>=QIUPDATEDELAY) then
			lQI.BGUpdate.last=0;
			QuestItems:BGUpdateItems();
		end
	end

	if (QuestItemsOptions.FadeFrame and not lQI.FrameFaded and lQI.FrameTimer>=0) then
		lQI.FrameTimer=lQI.FrameTimer-elapsed;
		if (lQI.FrameTimer<=0) then
			QuestItems:ShowFrame(nil);
		elseif (lQI.FrameTimer<=QUESTITEMS_FADETIME) then
			QuestItems:Lock(QuestItemsOptions.Locked,1-((QUESTITEMS_FADETIME-lQI.FrameTimer)/QUESTITEMS_FADETIME));
		end
	end
end

function QuestItems:SetNumber(button,item,num,r,g,b)
	if (not num) then num=""; end
	if (type(num)=="number" and num>=1000) then num=string.format("%.1fk",num/1000); end
	local buttontext=getglobal(button:GetName()..item);
	buttontext:SetText(num);
	buttontext:SetTextHeight(QuestItemsOptions.FontSize);
	buttontext:Show();
	if (not r and not g and not b) then return; end
	if (not r) then r=0; end
	if (not g) then g=0; end
	if (not b) then b=0; end
	buttontext:SetTextColor(r,g,b);
end

function QuestItems:SetItems(button)
	if (not button) then return nil; end
	self:UpdateButton(button);
end

function QuestItems:UpdateItems()
	lQI.BGUpdate.last=time();
	lQI.BGUpdate.xdir=1;
	lQI.BGUpdate.xpos=0;
	lQI.BGUpdate.ydir=1;
	lQI.BGUpdate.ypos=0;
end

function QuestItems:BGUpdateItems()
	while (lQI.BGUpdate.xdir~=0 or lQI.BGUpdate.ydir~=0) do
		local button=self:GetButton(lQI.BGUpdate.xpos,lQI.BGUpdate.ypos,true);		-- Do not create if non-existent
		if (button and not button.deleted) then
			self:SetItems(button);
			lQI.BGUpdate.xpos=lQI.BGUpdate.xpos+lQI.BGUpdate.xdir;
			return;
		end
		-- Check end of direction
		if (lQI.BGUpdate.xpos==0 and lQI.BGUpdate.xdir== 1 and lQI.BGUpdate.ydir== 1) then
			lQI.BGUpdate.xpos=-1; lQI.BGUpdate.xdir=-1; lQI.BGUpdate.ypos= 0; lQI.BGUpdate.ydir= 1;	-- One left to center, shift from left, aim up
		elseif (lQI.BGUpdate.xpos==0 and lQI.BGUpdate.xdir==-1 and lQI.BGUpdate.ydir== 1) then
			lQI.BGUpdate.xpos= 0; lQI.BGUpdate.xdir= 1; lQI.BGUpdate.ypos=-1; lQI.BGUpdate.ydir=-1;	-- One below center, shift from right, aim down
		elseif (lQI.BGUpdate.xpos==0 and lQI.BGUpdate.xdir== 1 and lQI.BGUpdate.ydir==-1) then
			lQI.BGUpdate.xpos= 0; lQI.BGUpdate.xdir=-1; lQI.BGUpdate.ypos=-1; lQI.BGUpdate.ydir=-1;	-- One below center, shift from left, aim down
		elseif (lQI.BGUpdate.xpos==0 and lQI.BGUpdate.xdir==-1 and lQI.BGUpdate.ydir==-1) then
			lQI.BGUpdate.xdir=0; lQI.BGUpdate.ydir=0; break;				-- Done
		else
			lQI.BGUpdate.xpos=0; lQI.BGUpdate.ypos=lQI.BGUpdate.ypos+lQI.BGUpdate.ydir;				-- Continue
		end
	end
	lQI.BGUpdate.last=nil;
end

function QuestItems:GetWindowPosition(x,y)
	local neg;
	neg=false; if (x<0) then neg=true; x=-x; end
	if (x>0) then x=QuestItemsOptions.Offset+((x-1)*(QuestItemsOptions.Size+QuestItemsOptions.Space))+QuestItemsOptions.Size/2; end
	if (neg==true) then x=-x; end
	neg=false; if (y<0) then neg=true; y=-y; end
	if (y>0) then y=QuestItemsOptions.Offset+((y-1)*(QuestItemsOptions.Size+QuestItemsOptions.Space))+QuestItemsOptions.Size/2; end
	if (neg==true) then y=-y; end
	return x,y;
end

function QuestItems:GetButton(x,y,nocreate,initfirstbutton)
	local dynName=QUESTITEMSBUTTON..x.."_"..y;
	local f=getglobal(dynName);
	if (not f or initfirstbutton) then
		if (nocreate) then return nil; end
		if (not initfirstbutton) then f=CreateFrame("Button",dynName,getglobal("QuestItems_Frame0"),"QuestItems_ButtonTemplate"); end
		self:SetButtonSize(f);
		f.deleted=true;
		f:SetPoint("CENTER",self:GetWindowPosition(x,y));
	end
	if (not nocreate or initfirstbutton) then f:Show(); end

	return f;
end

function QuestItems:Frame_OnEnter(which)
	local text="";
	if (not which) then return; end
	if (which==1) then
		text="Left-click and drag to move";
		if (QuestItemsOptions.Locked) then text=text.." (unlock window first)"; end
	elseif (which==2) then
		text="Right-click to open menu";
	else return; end
	GameTooltip:SetOwner(this,"ANCHOR_RIGHT");
	GameTooltip:SetText(text);
	GameTooltip:Show();
end

function QuestItems:SetButtonSize(button)
	button:SetWidth(QuestItemsOptions.Size);
	button:SetHeight(QuestItemsOptions.Size);

	getglobal(button:GetName().."MyCount"):SetTextHeight(QuestItemsOptions.FontSize);
	getglobal(button:GetName().."OurStock"):SetTextHeight(QuestItemsOptions.FontSize);
end

function QuestItems:SetSize(size)
	QuestItemsOptions.Size=size;
	QuestItemsOptions.Offset=math.ceil(QuestItemsOptions.Size/2)+QuestItemsOptions.Space;

	QuestItems_Frame0:SetWidth((QuestItemsOptions.Offset*2)+8);
	QuestItems_Frame0:SetHeight((QuestItemsOptions.Offset*2)+8);

	QuestItems_Frame0DropDownButtonT:SetWidth(QuestItemsOptions.Size*(0.7));
	QuestItems_Frame0DropDownButtonB:SetWidth(QuestItemsOptions.Size*(0.7));
	QuestItems_Frame0DropDownButtonL:SetHeight(QuestItemsOptions.Size*(0.7));
	QuestItems_Frame0DropDownButtonR:SetHeight(QuestItemsOptions.Size*(0.7));

	-- Center, shift from right, aim up
	local xdir=1;
	local xpos=0;
	local ydir=1;
	local ypos=0;

	while (xdir~=0 or ydir~=0) do
		while (self:GetButton(xpos,ypos,true)) do				-- Do not create if non-existent
			local button=self:GetButton(xpos,ypos);				-- Get the button
			local x,y=self:GetWindowPosition(xpos,ypos);
			button:SetPoint("CENTER",x,y);
			self:SetButtonSize(button);
			if (button.deleted) then button:Hide(); end
			xpos=xpos+xdir;
		end
		-- Check end of direction
			if (xpos==0 and xdir== 1 and ydir== 1) then xpos=-1; xdir=-1; ypos= 0; ydir= 1;	-- One left to center, shift from left, aim up
		elseif (xpos==0 and xdir==-1 and ydir== 1) then xpos= 0; xdir= 1; ypos=-1; ydir=-1;	-- One below center, shift from right, aim down
		elseif (xpos==0 and xdir== 1 and ydir==-1) then xpos= 0; xdir=-1; ypos=-1; ydir=-1;	-- One below center, shift from left, aim down
		elseif (xpos==0 and xdir==-1 and ydir==-1) then xdir=0; ydir=0; break;				-- Done
		else xpos=0; ypos=ypos+ydir; end													-- Continue
	end
end

function QuestItems:CreateButtons(width,height)
	if (not width) then width=QuestItemsOptions.Width; end
	if (not height) then height=QuestItemsOptions.Height; end
	local x,y=0,0;
	while (x<width) do
		while (y<height) do
			self:ClearItemButton(self:GetButton(x,y));		-- Make fresh button
			y=y+1;
		end
		y=0;
		x=x+1;
	end
end

function QuestItems:ShowFrame(showit)
	if (showit) then
		QuestItems:Lock(QuestItemsOptions.Locked,1);
		lQI.FrameTimer=QUESTITEMS_FRAMETIME;
		lQI.FrameFaded=nil;
	else
		QuestItems:Lock(QuestItemsOptions.Locked,0);
		lQI.FrameFaded=true;
	end
end

function QuestItems:Lock(lock,trans)
	if (lock==false) then lock=nil; end
	QuestItemsOptions.Locked=lock;
	self:SetCenter(trans);
end

function QuestItems:LockToggle()
	if (QuestItemsOptions.Locked) then QuestItems:Lock(nil); else QuestItems:Lock(true); end
	QuestItems:UpdateItems();
end

function QuestItems:SetCenter(trans)
	if (not trans) then _,_,_,trans=QuestItems_Frame0:GetBackdropBorderColor(); end
	if (QuestItemsOptions.Locked) then QuestItems_Frame0:SetBackdropBorderColor(1,0,0,trans);		-- R,G,B,A
	else QuestItems_Frame0:SetBackdropBorderColor(.5,.5,1,trans); end
end

-- There's slashing to be done
function QuestItems:Slasher(msg)
-- Validate input
	if (not msg) then msg=""; end
	if (strlen(msg)>0) then msg=strlower(msg); end

	if (msg=="tiny") then QuestItems:SetSize(20); return; end
	if (msg=="small") then QuestItems:SetSize(25); return; end
	if (msg=="medium") then QuestItems:SetSize(30); return; end
	if (msg=="normal") then QuestItems:SetSize(37); return; end
	if (msg=="big") then QuestItems:SetSize(45); return; end
	if (string.find(msg,"size ")==1) then
		val=string.sub(msg,5); val=tonumber(val);
		if (val) then QuestItems:SetSize((val*5)+20); end
		return;
	end

	QuestItems:Chat(QUESTITEMS_VERSIONTEXT);
	QuestItems:Chat(" Right-click the edge of the QuestItems frame for the menu.");
	QuestItems:Chat(" -> /qi tiny,small,medium,normal,big => Set predefined icon sizes");
	QuestItems:Chat(" -> /qi size X => Set any-sized icons with X=0,1,2,3 etc");
end

function QuestItems:BuildMenu()
	UIDropDownMenu_Initialize(QuestItems_Frame0Options,QuestItems.MenuLoad,"MENU");
end
function QuestItems:ShowMenu()
	ToggleDropDownMenu(1,nil,QuestItems_Frame0Options,this:GetParent(),0,0);
end
function QuestItems.MenuLoad()
	local info={};
	info.textR=1; info.textG=1; info.textB=1;
	if (QuestItemsOptions.Locked) then info.text="Unlock"; else info.text="Lock"; end
	info.func=QuestItems.LockToggle;
	UIDropDownMenu_AddButton(info,1);

	info.checked=QuestItemsOptions.FadeFrame;
	info.func=QuestItems.SetFrame;
	info.text="Fade center frame"; UIDropDownMenu_AddButton(info,1);
-- Width
	info.checked=nil;
	info.func=QuestItems.WidthChange;
	info.text="+ Width"; info.value=1; UIDropDownMenu_AddButton(info,1);
	info.text="- Width"; info.value=-1; UIDropDownMenu_AddButton(info,1);
-- Height
	info.checked=nil;
	info.func=QuestItems.HeightChange;
	info.text="+ Height"; info.value=1; UIDropDownMenu_AddButton(info,1);
	info.text="- Height"; info.value=-1; UIDropDownMenu_AddButton(info,1);

	info.checked=nil;
	info.func=QuestItems.TextSizeFromMenu;
	info.text="Bigger text"; info.value=1; UIDropDownMenu_AddButton(info,1);
	info.text="Smaller text"; info.value=-1; UIDropDownMenu_AddButton(info,1);
end

function QuestItems.WidthChange()
	QuestItems:TraverseIcons(QuestItems.HideButton,true);
	QuestItemsOptions.Width=QuestItemsOptions.Width+this.value;
	QuestItems:CreateButtons();
	lQI.Quest.Rescan=1;
end
function QuestItems.HeightChange()
	QuestItems:TraverseIcons(QuestItems.HideButton,true);
	QuestItemsOptions.Height=QuestItemsOptions.Height+this.value;
	QuestItems:CreateButtons();
	lQI.Quest.Rescan=1;
end
function QuestItems:WipeInterface()
	self:TraverseIcons(self.HideButton,true);
	self:CreateButtons();
end

function QuestItems:HideButton(button)
	button.deleted=true; button.itemID=nil; button.LastUpdated=nil;
	button:Hide();
	getglobal(button:GetName().."OurStock"):Hide();
	getglobal(button:GetName().."MyCount"):Hide();
end

function QuestItems:SetFrame()
	if (QuestItemsOptions.FadeFrame) then QuestItemsOptions.FadeFrame=nil; else QuestItemsOptions.FadeFrame=true; end
	QuestItems:ShowFrame(true);
end

function QuestItems:TextSizeFromMenu()
	QuestItemsOptions.FontSize=QuestItemsOptions.FontSize+this.value;
	QuestItems:SetSize(QuestItemsOptions.Size);
end

-- Tooltip-button mouseover
function QuestItems:Button_OnEnter()
	self:ShowFrame(true);
	self:Tooltip(this);
end

-- Generic chat-pane stuff
function QuestItems:Chat(msg,r,g,b)
	if (DEFAULT_CHAT_FRAME) then
		if (not r and not g and not b) then r=1; g=1; b=1; end;
		if (not r) then r=0; end;
		if (not g) then g=0; end;
		if (not b) then b=0; end;
		DEFAULT_CHAT_FRAME:AddMessage("QI: "..msg,r,g,b);
	end
end

-- Duh
function QuestItems:ShowMainFrame()
	if (not QuestItems_Frame0:IsVisible()) then
		QuestItems_Frame0:Show();
		self:ShowFrame(true);
	end
end
